@@ -19,5 +19,13 @@ api.register_callback("close", window.close_window)
 api.register_callback("bg", change_color)
 api.register_callback("testCallback", lambda: print("Button clicked!"))
 
+window1 = create_window(
+                "Test Window 1",
+                window_id="main_window",
+                html_file="C:\\Users\\SMILETECH COMPUTERS\\Documents\\pythra-toolkit\\src\\pythra\\pythra\\window\\ind.html",
+                js_api=api,
+                frameless=False, maximized=True, fixed_size=False)
+
 if __name__ == "__main__":
-    start(window=window, debug=True)
+    start(window=window, debug=False)
+    start(window=window1, debug=False)
